/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "power.h"

#ifdef QT_QWS_CUSTOM
#include "custom.h"
#endif

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#if defined(Q_OS_LINUX) || defined(_OS_LINUX_)
#include <sys/ioctl.h>
#include <unistd.h>
#endif

#ifdef QT_QWS_IPAQ_NO_APM
#include <linux/h3600_ts.h>
#endif

#include <qtopia/config.h>

PowerStatusManager2 *PowerStatusManager2::powerManager2 = 0;
PowerStatus2 *PowerStatusManager2::ps2 = 0;

PowerStatusManager2::PowerStatusManager2()
{
    powerManager2 = this;
    ps2 = new PowerStatus2;
}

const PowerStatus2 &PowerStatusManager2::readStatus()
{
    if ( !powerManager2 )
	(void)new PowerStatusManager2;

    powerManager2->getStatus();

    return *ps2;
}

// Standard /proc/apm reader
bool PowerStatusManager2::getProcApmStatus( int &ac, int &bs, int &bf, int &pc, int &sec )
{
    bool ok = false;

    ac = 0xff;
    bs = 0xff;
    bf = 0xff;
    pc = -1;
    sec = -1;

    FILE *f = fopen("/proc/apm", "r");
    if ( f ) 
	{
		//I 1.13 1.2 0x02 0x00 0xff 0xff 49% 147 sec
		char u;
		fscanf(f, "%*[^ ] %*d.%*d 0x%*x 0x%x 0x%x 0x%x %d%% %i %c",
			&ac, &bs, &bf, &pc, &sec, &u);
		fclose(f);
		switch ( u ) {
			case 'm': sec *= 60;
			case 's': break; // ok
			default: sec = -1; // unknown
		}

		switch ( bs ) 
		{
			case 0x00:
			ps2->bs = PowerStatus2::High;
			break;
			case 0x01:
			ps2->bs = PowerStatus2::Low;
			break;
			case 0x7f:
			ps2->bs = PowerStatus2::VeryLow;
			break;
			case 0x02:
			ps2->bs = PowerStatus2::Critical;
			break;
			case 0x03:
			ps2->bs = PowerStatus2::Charging;
			break;
			case 0x04:
			case 0xff: // 0xff is Unknown but we map to NotPresent
			default:
			ps2->bs = PowerStatus2::NotPresent;
			break;
		}

		switch ( ac ) 
		{
			case 0x00:
			ps2->ac = PowerStatus2::Offline;
			break;
			case 0x01:
			ps2->ac = PowerStatus2::Online;
			break;
			case 0x02:
			ps2->ac = PowerStatus2::Backup;
			break;
		}

		if ( pc > 100 )
			pc = -1;

		ps2->percentRemain = pc;
		ps2->secsRemain = sec;

		// extract data
		if (pc > 0 && (ac == PowerStatus2::Offline))
		{
			Config  cfg("qpe");
			cfg.setGroup("PowerLevels");

			int low =  cfg.readNumEntry("Low", 20);
			int veryLow =  cfg.readNumEntry("veryLow", 10);
			int critical =  cfg.readNumEntry("critical", 5);

			// default
			ps2->bs = PowerStatus2::NotPresent;

			// set status based on percentage
			if (pc > low)
				ps2->bs = PowerStatus2::High;

			if (pc <= low)
				ps2->bs = PowerStatus2::Low;

			if (pc <= veryLow)
				ps2->bs = PowerStatus2::VeryLow;

			if (pc <= critical)
				ps2->bs = PowerStatus2::Critical;
		}

		ok = true;
    }

    return ok;
}

// from custom.cpp
void PowerStatusManager2::getStatus()
{
    int ac, bs, bf, pc, sec;
    ps2->percentAccurate = TRUE; // not for long...

    if ( getProcApmStatus( ac, bs, bf, pc, sec ) ) {
	// special case
	if ( bs == 0x7f )
	    ps2->bs = PowerStatus2::VeryLow;
	//pc = -1; // fake percentage
	if ( pc < 0 ) {
	    switch ( bs ) {
		case 0x00: ps2->percentRemain = 100; break; // High
		case 0x01: ps2->percentRemain = 30; break; // Low
		case 0x7f: ps2->percentRemain = 10; break; // Very Low
		case 0x02: ps2->percentRemain = 5; break; // Critical
		case 0x03: ps2->percentRemain = -1; break; // Charging
	    }
	    ps2->percentAccurate = FALSE;
	}
    }

    char *device = "/dev/apm_bios";
    int fd = ::open (device, O_WRONLY);
    if ( fd >= 0 ) {
	int bbat_status = ioctl( fd, APM_IOC_BATTERY_BACK_CHK, 0 );
	switch ( bbat_status ) {
	    case 0x00:
		ps2->bbs = PowerStatus2::High;
		break;
	    case 0x01:
		ps2->bbs = PowerStatus2::Low;
		break;
	    case 0x7f:
		ps2->bbs = PowerStatus2::VeryLow;
		break;
	    case 0x02:
		ps2->bbs = PowerStatus2::Critical;
		break;
	    case 0x03:
		ps2->bbs = PowerStatus2::Charging;
		break;
	    case 0x04:
		ps2->bbs = PowerStatus2::NotPresent;
		break;
	}
	::close(fd);
    }
}

